"""
Copyright (c) 2014 Marta Nabozny

This file is part of OverCluster project.

OverCluster is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""


from overCluster.utils import request, calc_hash

from overCluster.models.template import Template
from overCluster.models.vm import VM
from overCluster.models.image import Image
from overCluster.models.network import Network

class Api():
    address = None
    token = None
    
    def __init__(self, address, token):
        self.token = token
        self.address = address


    def vm_list(self):
        vms = request(self.address, '/api/vm/get_list/', {'token': self.token})
        return [VM(self.address, self.token, vm) for vm in vms]


    def vm_create(self,
                  name,
                  description,
                  template,
                  base_image):
        vm = request(self.address, "/api/vm/create/", {'token': self.token,
                                                       'name': name,
                                                       'description': description,
                                                       'template_id': template.id,
                                                       'base_image_id': base_image.id})
        return VM(self.address, self.token, vm)


    def vm_by_id(self, id):
        vm = request(self.address, '/api/vm/get_by_id/', {'token': self.token,
                                                          'vm_id': id})
        return VM(self.address, self.token, vm)


    def image_list(self, type=None, access=None, prohibited_states=['deleted']):
        images = request(self.address, '/api/image/get_list/', {'token': self.token,
                                                                'type': type,
                                                                'access': access,
                                                                'prohibited_states': prohibited_states})
        return [Image(self.address, self.token, image) for image in images]


    def image_create(self, name, description, size, image_type, disk_controller, access):
        image = request(self.address, '/api/image/create/', {'token': self.token,
                                                             'name': name,
                                                             'description': description,
                                                             'size': size,
                                                             'image_type':image_type,
                                                             'disk_controller': disk_controller,
                                                             'access' : access})
        return Image(self.address, self.token, image)


    def image_by_id(self, id):
        image = request(self.address, '/api/image/get_by_id/', {'token': self.token,
                                                                'image_id': id})
        return Image(self.address, self.token, image)


    def supported_disk_controllers(self):
        r = request(self.address, '/api/image/get_disk_controllers/', {'token': self.token})
        return r


    def supported_video_devices(self):
        r = request(self.address, '/api/image/get_video_devices/', {'token': self.token})
        return r


    def supported_network_devices(self):
        r = request(self.address, '/api/image/get_network_devices/', {'token': self.token})
        return r


    def supported_image_types(self):
        r = request(self.address, '/api/image/get_image_types/', {'token': self.token})
        return r


    def template_list(self):
        templates = request(self.address, '/api/template/get_list/', {'token': self.token})
        return [Template(self.address, self.token, template) for template in templates]


    def network_request(self, mask, name, isolated=False):
        n_dict = request(self.address, '/api/network/request/', {'token': self.token,
                                                                 'mask': mask,
                                                                 'name': name,
                                                                 'isolated': isolated})
        return Network(self.address, self.token, n_dict)


    def network_by_id(self, id):
        n_dict = request(self.address, '/api/network/get_by_id/', {'token': self.token,
                                                                 'network_id': id})
        return Network(self.address, self.token, n_dict)


    def network_list(self):
        networks = request(self.address, '/api/network/list_user_networks/', {'token': self.token})
        return [Network(self.address, self.token, network) for network in networks]